Add command line option to set machine check tolerance level

Signed-off-by: Andi Kleen <ak@suse.de>

Index: linux/Documentation/x86_64/boot-options.txt
===================================================================
--- linux.orig/Documentation/x86_64/boot-options.txt
+++ linux/Documentation/x86_64/boot-options.txt
@@ -11,6 +11,11 @@ Machine check
                If your BIOS doesn't do that it's a good idea to enable though
                to make sure you log even machine check events that result
                in a reboot.
+   mce=tolerancelevel (number)
+		0: always panic, 1: panic if deadlock possible,
+		2: try to avoid panic, 3: never panic or exit (for testing)
+		default is 1
+		Can be also set using sysfs which is preferable.
 
    nomce (for compatibility with i386): same as mce=off
 
Index: linux/arch/x86_64/kernel/mce.c
===================================================================
--- linux.orig/arch/x86_64/kernel/mce.c
+++ linux/arch/x86_64/kernel/mce.c
@@ -17,6 +17,7 @@
 #include <linux/fs.h>
 #include <linux/cpu.h>
 #include <linux/percpu.h>
+#include <linux/ctype.h>
 #include <asm/processor.h> 
 #include <asm/msr.h>
 #include <asm/mce.h>
@@ -489,6 +490,7 @@ static int __init mcheck_disable(char *s
 
 /* mce=off disables machine check. Note you can reenable it later
    using sysfs.
+   mce=TOLERANCELEVEL (number, see above)
    mce=bootlog Log MCEs from before booting. Disabled by default to work
    around buggy BIOS that leave bogus MCEs.  */
 static int __init mcheck_enable(char *str)
@@ -499,6 +501,8 @@ static int __init mcheck_enable(char *st
 		mce_dont_init = 1;
 	else if (!strcmp(str, "bootlog"))
 		mce_bootlog = 1;
+	else if (isdigit(str[0]))
+		get_option(&str, &tolerant);
 	else
 		printk("mce= argument %s ignored. Please use /sys", str); 
 	return 0;
